local T = ...

T.Tab = {
	title = "Asteroids",
	columns = {
		{
			title = "ID",
			width = 0,
		},
		{
			title = "Ore",
			width = 0.8,
		},
		{
			title = "Distance",
			width = 0.2,
		},
	},
	filters = {
		{
			title = "Ore",
		},
		{
			title = "Aq",
			pages = {},
		},
		{
			title = "Unscanned",
		},
	},
	-- -------------------
	current_target_list_index,
	filter_index = -1,
	filter_ore_type,
	filter_page = {0, 0},
	is_active = false,
	roid_list = {},
	Scan = {
		full_freq = 15,
		sub_freq = 5,
		sub_max = 100,
		subsub_max = 25,
		cycle_count = 15,  -- force full_scan on first run.
		interval = 1000,
		full,
		timer = Timer(),
	},
}
-- populate the 2nd filter pages
for i, o in ipairs(T.ore_reference[1]) do
	T.Tab.filters[2].pages[i] ={title = o}
end

function T.Tab:ChangeTarget(oldID, newID)
	local roid_list = T.Tab.roid_list
	if (roid_list[newID]) then
		roid_list[newID].highlight = true
		if (oldID and oldID ~= newID) then
			roid_list[oldID].highlight = nil
		end
		T.Scan.LockFunction(function() radar.SetRadarSelection(T.CurrentSectorNodeID, roid_list[newID][1]) end)
		self.current_target_list_index = newID
	end
	targetlist.API.UpdateDisplay(self, roid_list)
end

function T.Tab:ChangeFilter(index, page)
	self.filter_index = index

	if (index == 1) then
		local r, num_filter_pages = pcall(function() return #self.filters[index + 1].pages end)
		local num_filter_pages = (r and num_filter_pages or 1)
		self.filter_ore_type = self.filters[index + 1].pages[page + 1].title
		self.filters[index + 1].title = self.filter_ore_type
		targetlist.API.UpdateFilter(self.filter_ore_type)
	else
		self.filter_ore_type = nil
	end

	self.Scan.cycle_count = self.Scan.full_freq -- force full scan on start
	self:HUD_SHOW(nil, "ship")
end

function T.Tab:TARGETLIST_TAB_CHANGED()
 	self.is_active = targetlist.API.IsActiveTab(self)
	if (self.is_active) then
		RegisterEvent(self, "TARGET_CHANGED")
		RegisterEvent(self, "WEAPON_PROGRESS_START")
		RegisterEvent(self, "WEAPON_PROGRESS_STOP")
	else
		UnregisterEvent(self, "TARGET_CHANGED")
		UnregisterEvent(self, "WEAPON_PROGRESS_START")
		UnregisterEvent(self, "WEAPON_PROGRESS_STOP")
		self.Scan.timer:Kill()
	end
end

function T.Tab:TARGET_CHANGED(_, data)
	if (T.Scan.Lock()) then return end

	local new_target_nodeID, new_target_objectID = radar.GetRadarSelectionID()

	for i, t in ipairs(self.roid_list) do
		t.highlight = new_target_nodeID == T.CurrentSectorNodeID and t.id == new_target_objectID
	end
	self:ChangeTarget()
end

-- HUD_SHOW is a more consistent event to trigger the start of the scan with.
function T.Tab:HUD_SHOW(_, hud)
	if(self.is_active and hud == "ship") then
-- TODO Figure out why the tab scanners aren't restarting properly or clearing the existing list.
		self.roid_list = {}
		if (self.filter_index == 2) then
			T.Scan.LockFunction(T.Scan.UnscannedRoids)
		else
			T.Scan.LockFunction(T.Scan.ScannedRoids)
		end
	end
end

function T.Tab:HUD_HIDE(_, event, data)
	if (self.is_active) then
		self.Scan.timer:Kill()
	end
end

function T.Tab:PLAYER_ENTERED_GAME()
	--self:TARGETLIST_TAB_CHANGED()
	if (not (isdeclared("targetlist") and targetlist.API)) then
		print("\127FFFFFFThe MDI-RoidDB Asteroids Tab requires MDI-TargetList v4+.\127o")
	end
end

function T.Tab:PLAYER_LOGGED_OUT()
	self.Scan.timer:Kill()
end

function T.Tab:PLUGINS_LOADED()
	if (isdeclared("targetlist") and targetlist.API) then
		if (targetlist.API.Version >= 1) then
			targetlist.API.RegisterTab(T.Tab)
		else
			print("\127FFFFFFThe MDI-TargetList API version is different than expected.  MDI-RoidDB Tab was not loaded.\127o")
		end
	end
end

function T.Tab:WEAPON_PROGRESS_START()
	self.Scan.timer:Kill()
end

function T.Tab:WEAPON_PROGRESS_STOP()
	self.Scan.timer:SetTimeout(self.Scan.interval, function() self:HUD_SHOW(nil, "ship") end)
end


RegisterEvent(T.Tab, "PLUGINS_LOADED")
RegisterEvent(T.Tab, "PLAYER_ENTERED_GAME")
RegisterEvent(T.Tab, "PLAYER_LOGGED_OUT")
RegisterEvent(T.Tab, "TARGETLIST_TAB_CHANGED")
RegisterEvent(T.Tab, "HUD_HIDE")
RegisterEvent(T.Tab, "HUD_SHOW")
--RegisterEvent(T.Tab, "TARGET_CHANGED") -- Only registered when the tab is active
--RegisterEvent(T.Tab, "WEAPON_PROGRESS_START")  -- Only registered when the tab is active
--RegisterEvent(T.Tab, "WEAPON_PROGRESS_STOP")  -- Only registered when the tab is active


local function ProcessUnscannedRoids(sector_data, roid_list, id, orig_targetID, full_scan)
	local roidID = (full_scan and id or roid_list[id][1]) -- "id" could be either the roidID or the listID.
	radar.SetRadarSelection(T.Scan.GetSectorNodeID(), roidID) -- don't ScanLock here.  It's already inside of a ScanLock

	-- Necessary to make sure we actually changed roids.  Won't change if not in range.
	local _, new_roidID = radar.GetRadarSelectionID()
	if (new_roidID == roidID and T.RoidNames[GetTargetInfo()]) then
		if (not sector_data[roidID]) then
			-- If the current targeted roid has not yet been scanned
			local distance = math.floor(GetTargetDistance())
			local ore_info = ""

			if (full_scan) then
				table.insert(roid_list, {roidID, "Asteroid #"..roidID, distance, highlight = roidID == orig_targetID})
			else
				roid_list[id][3] = distance
				roid_list[id]["highlight"] = roidID == orig_targetID
			end

		else --if (id ~= roidID) then
			table.remove(roid_list, id)
		end
	end
end

function T.Scan.UnscannedRoids()
	-- don't scan if an auto-scan is in progress
	if (T.Scan.ningAuto) then
		T.Tab.roid_list = {{-1,"Auto-Scan in progress"}}
		T.Tab.Scan.cycle_count = T.Tab.Scan.full_freq

	elseif (T.Scan.GetSectorNodeID()) then -- don't ScanLock here.  It's already inside of a ScanLock
		local sector_data = T.GetData(GetCurrentSystemid(), GetCurrentSectorid())

		local orig_nodeID, orig_targetID = radar.GetRadarSelectionID()
		gkinterface.GKProcessCommand("RadarNone")

		-- ----------------------------------------------------
		local cur_count, max_count = 0, 0
		local full_scan = false

		if (T.Tab.Scan.cycle_count >= T.Tab.Scan.full_freq) then
			-- Full Scan
			full_scan = true
			T.Tab.Scan.cycle_count = 0
			T.Tab.roid_list = {}

		elseif ((T.Tab.Scan.cycle_count % T.Tab.Scan.sub_freq) == 0) then
			-- Sub Scan
			max_count = #T.Tab.roid_list <= T.Tab.Scan.sub_max and #T.Tab.roid_list or T.Tab.Scan.sub_max

		else
			-- Sub Sub Scan
			max_count = #T.Tab.roid_list <= T.Tab.Scan.subsub_max and #T.Tab.roid_list or T.Tab.Scan.subsub_max
		end
		-- ----------------------------------------------------

		local roid_list = T.Tab.roid_list

		if (full_scan) then
			for roidID = 1, 20000 do
				ProcessUnscannedRoids(sector_data, roid_list, roidID, orig_targetID, full_scan)
			end

		else
			for listID, roidtbl in ipairs(roid_list) do
				ProcessUnscannedRoids(sector_data, roid_list, listID, orig_targetID)
				cur_count = cur_count + 1
				if (cur_count == max_count) then break end
			end
		end

		table.sort(roid_list, function(a, b)
			return a[3] < b[3] -- distance index
		end)

		-- Retarget our original target
		if (orig_nodeID) then
			radar.SetRadarSelection(orig_nodeID, orig_targetID)
		else
			gkinterface.GKProcessCommand("RadarNone")
		end

	end

	targetlist.API.UpdateDisplay(T.Tab, T.Tab.roid_list)
	T.Tab.Scan.cycle_count = T.Tab.Scan.cycle_count + 1
	T.Tab.Scan.timer:SetTimeout(T.Tab.Scan.interval, function() T.Scan.LockFunction(T.Scan.UnscannedRoids) end)
end

function T.Scan.ScannedRoids()
	-- don't scan if an Auto-scan is in progress
	if (T.Scan.ningAuto) then
		T.Tab.roid_list = {{-1,"Auto-Scan in progress"}}
		T.Tab.Scan.cycle_count = T.Tab.Scan.full_freq

	-- only try to scan if we've got a nodeid for the sector
	elseif (T.Scan.GetSectorNodeID()) then -- don't ScanLock here.  It's already inside of a ScanLock
		-- Save the original target
		local orig_nodeID, orig_targetID = radar.GetRadarSelectionID()
		gkinterface.GKProcessCommand("RadarNone")

		-- ---------------------------------------------------------
		-- Determine the level of scan/update being performed (full, sub, subsub)
		local cur_count, max_count = 0, 0
		local scan_data = T.Tab.roid_list -- Set scan_data to the current roid_list since the primary scan will be an update.
		local full_scan = false
		if (T.Tab.Scan.cycle_count >= T.Tab.Scan.full_freq) then
			-- Full Scan
			full_scan = true
			T.Tab.Scan.cycle_count = 0
			local sector_data = T.GetData(GetCurrentSystemid(), GetCurrentSectorid())
			for _ in pairs(sector_data) do max_count = max_count + 1 end  -- TODO make this less clumsy TODO
			scan_data = sector_data -- Need to refresh the scanned roids list
			T.Tab.roid_list = {}

		elseif ((T.Tab.Scan.cycle_count % T.Tab.Scan.sub_freq) == 0) then
			-- Sub Scan
			max_count = #T.Tab.roid_list <= T.Tab.Scan.sub_max and #T.Tab.roid_list or T.Tab.Scan.sub_max

		else
			-- Sub Sub Scan
			max_count = #T.Tab.roid_list <= T.Tab.Scan.subsub_max and #T.Tab.roid_list or T.Tab.Scan.subsub_max
		end
		-- ---------------------------------------------------------

		-- cycle through scanned roids.  populate the roid_list with all roids in range or update the distances of those already scanned
		for roidID, roidtbl in pairs(scan_data) do
			roidID = full_scan and roidID or roidtbl[1]
			radar.SetRadarSelection(T.Scan.GetSectorNodeID(), roidID) -- don't ScanLock here.  It's already inside of a ScanLock

			-- Necessary to make sure we actually changed roids.  Won't change if not in range.
			local _, new_roidID = radar.GetRadarSelectionID()
			if (new_roidID == roidID) then
				local distance = math.floor(GetTargetDistance())
				local ore_info = ""
				local insert_into = not full_scan

				-- Populate the ore info string for display
				if (full_scan) then
					for _, ore_type in ipairs(T.ore_reference[1]) do
						local ore_pct = roidtbl.ore[ore_type]
						if (ore_pct) then
							local ore_matches_filter = ore_type == T.Tab.filter_ore_type
							insert_into = insert_into or ore_matches_filter -- only flag roids for insert if they match the current filter
							ore_info = ore_info..(ore_matches_filter and "\127FF0000" or "")..ore_type..":"..ore_pct.."\127o "
						end
					end
				end

				-- Insert into or Update the roid list.
				if (insert_into  or T.Tab.filter_index == 0) then -- override the insert flag if on the first filter
					if (full_scan) then
						table.insert(T.Tab.roid_list, {roidID, ore_info, distance, highlight = roidID == orig_targetID})
					else
						roidtbl[3] = distance
						roidtbl["highlight"] = roidID == orig_targetID
					end
				end
			end
			cur_count = cur_count + 1
			if (cur_count == max_count) then break end
		end

		table.sort(T.Tab.roid_list, function(a, b)
			if (T.Tab.filter_ore_type) then
				local A = (tonumber(a[2]:match(T.Tab.filter_ore_type..":([%w.]+)")) or 0)
				local B = (tonumber(b[2]:match(T.Tab.filter_ore_type..":([%w.]+)")) or 0)

				return A > B or (A == B and a[3] < b[3]) or false
			end
			return a[3] < b[3] -- distance index
		end)

		-- Retarget our original target
		if (orig_nodeID) then
			radar.SetRadarSelection(orig_nodeID, orig_targetID)
		else
			gkinterface.GKProcessCommand("RadarNone")
		end

	end
	targetlist.API.UpdateDisplay(T.Tab, T.Tab.roid_list)
	T.Tab.Scan.cycle_count = T.Tab.Scan.cycle_count + 1
	T.Tab.Scan.timer:SetTimeout(T.Tab.Scan.interval, function() T.Scan.LockFunction(T.Scan.ScannedRoids) end)
end

