local T = ...

local function get_control_width(str)
	local text = iup.label{title = "000.0", expand = "NO"}
	iup.Map(text)
	str = tonumber(text.w)
	iup.Destroy(text)
	return str
end

local function check_filters(data, filter)
	if (filter == "All" or filter == nil) then
		return true
	elseif (data[filter]) then 
		return true 
	end
	return false
end

local function tally_scanned_roids(systemid, sectorid)
	local count = 0
	local max = 0
	for i in pairs(T.GetData(systemid, sectorid)) do
		count = count + 1
		if (i > max) then max = i end
	end
	T.UI.scanned_total.title = count
	T.UI.est_max_total.title = max
end


-- ##############################################################################
-- ##############################################################################
-- ##############################################################################

function T:BuildNavmap()
	local navmap = iup.navmap{bgcolor = "0 255 0 255"}
	local info_display  = iup.multiline{expand = "YES", active = "NO", font = T.Font, size = T.IsMobile() and "" or "x%15"}
	local unknown = iup.stationbutton{title = "Unknown System", font = T.Font, visible = "NO"}


	local container_content = {
		iup.pdarootframe{
			iup.zbox{
				navmap,
				unknown,
				all = "YES",
				alignment = "NW"
			},
		},
		iup.pdarootframe{
			info_display,
		},
		gap = 10,
	}

	local navmap_container = T.IsMobile() and iup.hbox(container_content) or iup.vbox(container_content)

-- ##############################################################################
-- ##############################################################################
-- ##############################################################################

	local mapmode = 1
	local cur_systemid
	local cur_sectorid
	local cur_filter

	function navmap:GetMapMode() return mapmode end
	function navmap:GetSectorID() return cur_sectorid end
	function navmap:GetSystemID() return cur_systemid end

	function navmap:SetInfo(str) info_display.value = str end
	function navmap:SetMapMode(mode) mapmode = mode > 1 and 2 or 1 end
	function navmap:SetSectorID(id) 
		cur_sectorid = id 
		if (mapmode == 2) then 
			self.clickedid = cur_sectorid 
		end
	end
	function navmap:SetSystemID(id) 
		cur_systemid = id 
		if (mapmode == 1) then self.clickedid = cur_systemid - 1 end
	end
	
	function navmap:click_cb(index, flags, refresh)
		if (not index) then return end
		if (not refresh) then
			T.UI.scanned_total.title = "-"
			T.UI.est_max_total.title = "-"
			if (mapmode == 1) then
				cur_systemid = index + 1 -- convert system indexes into 1-based indexes for general use.
				cur_sectorid = nil -- clear sectorid, since the old one no longer applies to the selected sytemid
				if (flags) then
					if (string.byte(flags or "", 16) == iup.K_L) then -- Long-click/press
						-- support long click?  do what?
						return
					end
					unknown.bgcolor = "255 255 255 255 *"
				else
					-- Do I need to do anything if this is called by unknown:action() or refresh()?
				end

			else
				T.UI.setpathbtn.active = IsConnected() and "YES" or "NO"

				if (index == cur_sectorid) then
					--self.clickedid = 0
					cur_sectorid = nil
					self:refresh()
				else
					cur_sectorid = index
					-- update scanned totals
					tally_scanned_roids(cur_systemid, cur_sectorid)
				end
				T.UI.nextbtn.active = IsConnected() and cur_sectorid and "YES" or "NO"
			end
		end
		T.UI.filters_and_highest:SetHighest(cur_systemid, cur_sectorid)
	end

	function navmap:refresh(filter)
		if     (mapmode == 1) then
			self:loadmap(1, "lua/maps/universemap.lua", 0)

			-- All navmap related uses of systemid are 0-based.
			self.clickedid =  cur_systemid and cur_systemid - 1 -- need the extra check incase it is nil and we try to math it
			self.currentid = IsConnected() and GetCurrentSystemid() - 1
			unknown.visible = next(T.GetData(33)) and "YES" or "NO"
			if (cur_systemid == T.HighestSystemID) then unknown.bgcolor = "255 0 0" end

		elseif (mapmode == 2) then
			-- if the asteroid dataset has Unknown System information, but the player is not in the Unknown System, the map won't display.
			-- adjust the cur_systemid if necessary to display a map.
			local adjusted_systemid = cur_systemid
			if (adjusted_systemid == T.HighestSystemID and GetCurrentSystemid() ~= adjusted_systemid) then
				adjusted_systemid = 16 -- Pelatus
			end
			self:loadmap(2, string.format("lua/maps/system%02dmap.lua", adjusted_systemid), cur_systemid - 1)
			-- color the sectors with ore in them
			for i, sectortbl in pairs(T.GetSystemSummary(cur_systemid)) do
				local highest = sectortbl[filter] and sectortbl[filter].highest or sectortbl.highest or 100
				if (next(sectortbl) and check_filters(sectortbl, filter) and highest >= (tonumber(T.UI.search_min.value) or 0)) then
					self["color"..i] = "56 255 55"
				end
			end
			-- attempt to color all of the training sectors regardless of system being viewed
			if (IsConnected()) then
				for i = 1, 3 do
					for _, sectorid in ipairs(GetStartingSectors(i)) do
						self["color"..sectorid] = "255 55 55"
					end
				end
			end

			-- All navmap related uses of sectorid are 1-based.
			self.clickedid = cur_sectorid
			self.currentid = IsConnected() and GetCurrentSectorid()
			unknown.visible = "NO"

		end

		if (self.clickedid) then 
			self:click_cb(tonumber(self.clickedid), nil, true)
		else
			T.UI.filters_and_highest:SetHighest(cur_systemid)
		end
		if (IsConnected()) then
			self:setpath(GetFullPath(GetCurrentSectorid(), NavRoute.GetCurrentRoute()))
		end
		cur_filter = filter or cur_filter
	end

	function navmap:reset(filter)
		self:refresh(filter)
	end

	function navmap:mouseover_cb(index, str)
		if (mapmode == 2) then
			if (cur_systemid == T.HighestSystemID and GetCurrentSystemid() ~= cur_systemid) then
				str = string.format("\127FF0000This is not the official map of the Unknown System.\127o\nUnknown System, "..str:match("Sector %w%-%d+"))
			else
				str = (str..(GetBotSightedInfoForSector(index) or "")):gsub("|", "\n")
			end
		end
		self:SetInfo(str)
	end

	function unknown:action()
		navmap.clickedid = 33 - 1
		navmap:click_cb(tonumber(navmap.clickedid))
		self.bgcolor = "255 0 0"
		navmap:SetInfo("Unknown System")
	end

	return navmap_container, navmap, unknown
end


-- navmap{size, expand, scrollbar}

-- navmap uses 0-based systemids.  all systemid inputs/outputs for navmap methods and events will be this way.  elsewhere systemids are 1-based 

-- navmap:loadmap(mode, mappath, page)
-- 	mode: 1 or 2;  1 = gridless; 2 = grid mode
-- 	mappath: "/lua/maps/(navmap.universemap.lua | system%02dmap.lua)"
-- 	page: systemid - 1; sets the page for sector ids.  start index (upper left) == 1 + (256 * page)

-- navmap:setpath(GetFullPath())

-- navmap.currentid = <id>; universe map: values are systemid-1; system map: sectorid range determined by the map page
-- navmap.clickedid = <id>; moves the clicked sector.  universe map: values are systemid-1;  system map; range determined by the map page

-- navmap.["color"..number]   does not work in mode 1; number is the sectorid.  available sector ids are determined by the current page;  changing the page resets all colors;  changing back to a previous page does not show previous colors



-- if no mappoint is selected,
-- 	tab will navigate in tab order to other controls



-- function navmap:k_any(c)
-- 	key presses repeat so long as the key is held down
-- 	ctrl does not repeat, but the combination does.
-- 	no character code for combinations.  in a ctrl combo, ctrl only triggers once and the other key triggers repeatedly
-- 	arrows trigger mouseover_cb, not k_any.
-- 	enter triggers click_cb
-- 	space triggers click_cb

-- keys
-- 	ctrl triggers, but does not repeat
-- 	shift does not trigger
-- 	alt triggers; can't combine with a mouse click on linux
-- end
-- 
-- function navmap:click_cb(index, flags)
-- 	only triggers when clicking on a mappoint, on mouseup
-- 	pressing enter does not set a flag
-- 	pressing space does not set a flag
-- 	in mode 2, if the long-press timeout is met, the callback finishes, not waiting for the button release.

-- 	flags "SC1            L "
-- 		1: S (shift)
-- 		2: C (ctrl)
-- 		3: 1 (left mouse click)
-- 		16: L (long-press 1 second(?))

-- 	use string.byte() to test for a specific flag
-- end
-- 
-- function navmap:getfocus_cb()
-- 	triggers:
-- 		when opening the navmap's dialog 
-- 		when pressing escape if the last action was a click
-- 	After this callback has triggered, all keyboard presses will trigger :k_any()
-- end
-- 
-- function navmap:mouseover_cb(index, str)
-- 	triggers when 
-- 		mousing over a map point.  nothing happens when moving the mouse over anything else
-- 		pressing arrow keys after a mappoint has been selected; order of system traversal on the universe map is in systemid order
-- 		pressing tab when a mappoint has been clicked on;  triggers the mouseover for the active mappoint

-- end


